import requests
import logging

from mmwebreport.core.response import Response


class Executor(object):
    """
    A class used to represent a Http request to the Monitor Manager Web Report Backend
    """
    def __init__(self, uri):
        self._uri = uri

    def run(self):
        """
        Make a Http request to the Monitor Manager Web Report Backend and return a Response object. In case
        of HTTP request error it returns an exception.
        """
        logging.info(self._uri)
        r = requests.get(self._uri)
        if r.status_code == 200:
            return Response(r)
        else:
            # todo add more error conditions
            raise requests.exceptions.HTTPError



import re
import json


class Response(object):
    """
    A class used to represent a response from the Monitor Manager Web Report Backend
    """
    def __init__(self, r):
        self._r = r

    def _parse_raw_text(self, text: object) -> object:
        """
        Parse the raw text coming from report request to the Monitor Manager Web Report Backend

        :param text:

                The raw format is:

            totalSamples    totalDisplaySamplesByPages  totalPages  page[X-Y]
            <number>        <number>                    <number>    <number>
            <blank line>
            TimeStamp       TimeStampLong   [Monitor1(Unit)]    [Monitor2(Unit)]    ... [MonitorN(Unit)]
            <date1>         <number>       <number>             <number>            ... <number>
            ...
            <date(n)>       <number>       <number>             <number>            ... <number>

        :return:

                Two variable:
                
            - header: The head of the report response:
            
                TimeStamp       TimeStampLong   [Monitor1(Unit)]    [Monitor2(Unit)]    ... [MonitorN(Unit)]
                
            - body: The body of the report response
            
                <date1>         <number>       <number>             <number>            ... <number>
                ...
                <date(n)>       <number>       <number>             <number>            ... <number>
                
        """

        text = text.split('\n')

        text_header = ','.join(text[3].replace("/", ".").split(",")[1:])
        # todo extract all metadata
        text_header = re.sub(r"\([^)]*\)", "", text_header)

        text_body = [','.join(line.split(",")[1:]) for line in text[4:]]
        text_body = '\n'.join(text_body)

        return text_header, text_body

    def to_csv(self):
        """
        Convert the raw text coming from report request to the Monitor Manager Web Report Backend to csv format
        """
        header, body = self._parse_raw_text(self._r.text)

        return header + '\n' + body

    def to_json(self):
        """
        Convert the raw text coming from report request to the Monitor Manager Web Report Backend to json format
        """
        return json.loads(self._r.text)



from datetime import datetime, timedelta

import pandas as pd
import logging

def _remove_similar_consecutive_values(data_frame, monitor_name, epsilon):

    if data_frame.size > 0:
        to_remove = []

        pivot = data_frame.loc[0][1:]
        for idx, row in data_frame.iterrows():
            if idx == 0:
                continue
            if ((pivot - row[1:]) ** 2).sum() ** 0.5 <= epsilon:
                to_remove.append(idx)
            else:
                pivot = row

        data_frame.drop(to_remove, axis=0, inplace=True)

    return data_frame

#def _remove_similar_consecutive_values(data_frame, monitor_name, epsilon):
    """
    Give a data frame it removes the values consecutive that are similar.

    :param data_frame: a data frame with value to be filtered
    :param monitor_name: a monitor to be filtered in the data frame
    :param epsilon: the epsilon to check if two values are similar

    :return: a data frame where similar values, base on epsilon, were removed.
    """
    # if data_frame.size > 0:
    #     to_remove = []
    #
    #     pivot = data_frame[monitor_name][0]
    #     for idx, row in data_frame.iterrows():
    #         if idx == 0:
    #             continue
    #         if abs(pivot - row[monitor_name]) < epsilon:
    #             to_remove.append(idx)
    #         else:
    #             pivot = row[monitor_name]
    #
    #     data_frame.drop(to_remove, axis=0, inplace=True)
    #
    # return data_frame


def _convert(data_frame):
    data_frame['TimeStampLong'] = pd.to_datetime(data_frame['TimeStampLong'], unit='us')

    return data_frame


def _merge_data_frames(data_frames):

    data_frame = data_frames[0]
    if len(data_frames) >= 2:
        data_frame = pd.merge(data_frames[0], data_frames[1], how='outer')
        for idx in range(2, len(data_frames)):
            data_frame = pd.merge(data_frame, data_frames[idx], how='outer')

    # data_frame['TimeStampLong'] = pd.to_datetime(data_frame['TimeStampLong'], unit='us')
    data_frame.sort_values(by=['TimeStampLong'], inplace=True)

    return data_frame


def _filter(data_frame, q_data_ini, q_data_end):

    data_frame = data_frame[
        (data_frame['TimeStampLong'] >= q_data_ini) &
        (data_frame['TimeStampLong'] < q_data_end) ]

    return data_frame


#todo method not working
def _make_time_intervals(q_data_ini, q_data_end):
    """
    It creates a list of hourly time interval between the date initial and date final. For example:

    Given:

        2022-03-01 22:30 - 2022-03-02 01:15

    It creates the following intervals:

        2022-03-01  22:00   -   2022-03-01  23:00
        2022-03-01  23:00   -   2022-03-02  00:00
        2022-03-02  00:00   -   2022-03-02  01:00
        2022-03-02  01:00   -   2022-03-02  02:00

        2022-03-02  22:00   -   2022-03-02  23:00
        2022-03-02  23:00   -   2022-03-03  00:00
        2022-03-03  00:00   -   2022-03-03  01:00
        2022-03-03  01:00   -   2022-03-03  02:00

    :return: a list of time intervals
    """
    time_intervals = []

    num_days = (q_data_end - q_data_ini).days + 1
    for day in range(0, num_days):
        hour_pivot = q_data_ini.hour
        time_end_hour = q_data_end.hour + (
            0 if q_data_end.minute == 0 and q_data_end.second == 0 else 1)
        while (hour_pivot % 24) != time_end_hour:
            a_interval_ini = datetime(q_data_ini.year,
                                      q_data_ini.month,
                                      q_data_ini.day,
                                      hour_pivot % 24, 0, 0)
            a_interval_ini += timedelta(days=day + int(hour_pivot / 24.0))

            hour_pivot += 1

            a_interval_end = datetime(q_data_ini.year,
                                      q_data_ini.month,
                                      q_data_ini.day,
                                      hour_pivot % 24, 0, 0)
            a_interval_end += timedelta(days=day + int(hour_pivot / 24.0))

            time_intervals.append((a_interval_ini, a_interval_end))

    return time_intervals



import os
import io

from os.path import exists

import pandas as pd
import logging

from mmwebreport.core.api import Report
from mmwebreport.retrieve import cache
from mmwebreport.retrieve.processing import _remove_similar_consecutive_values, _convert, _merge_data_frames, _filter, \
    _make_time_intervals


class RetrieveMonitor(object):
    """
    A class used to make a high level request to the Monitor Manager Web Report, with the following functionality:
    todo
    """

    def __init__(self, host, port, q_query, q_name):
        self.request = Report(host, port)
        self._query = q_query
        self._query_name = q_name
        self._path = os.path.expanduser("~") + "/.cache/webreport/monitormanager"

    def retrieve_raw(self, date_ini, date_end, q_entry):
        """
        Given the time interval and a query entry create a data frame with the result. The request result
        is cached in disk.

        :param q_entry: todo
        :param date_ini: The search request initial date and time.
        :param date_end: The search request end date and time.
        :param monitor: The search request query entry.
            {
                "component": "MACS.AzimuthAxis",
                "monitor": "position",
                "epsilon": 0.5,
                "type": "monitor"
            }

        :return: a data frame for this request, filtering the similar values base on epsilon monitor param.
        """
        cursor = self.request.search(date_ini, date_end, [q_entry])

        monitor_key = q_entry['component'] + "." + q_entry['monitor']

        data_frames_page = []
        for page, c in enumerate(cursor):
            logging.info("Retrieve monitor %s from %s to %s page %s", monitor_key, date_ini, date_end, page)
            path = cache.make_path_raw(self._path, date_ini, date_end, monitor_key, page)
            logging.info("Retrieve monitor path %s ", os.path.dirname(path))
            if not exists(os.path.dirname(path)):
                os.makedirs(os.path.dirname(path))
            if exists(os.path.dirname(path) + "/query" + str(page) + ".json") and \
                    cache.compare_raw_query(cache.build_query(date_ini, date_end, self._query_name, [q_entry], page),
                                            cache.read_query(os.path.dirname(path), page)):
                logging.info("Retrieve monitor from cache %s ", path)
                data_frame = pd.read_csv(path, compression='infer')
            else:
                data_frame = pd.read_csv(io.StringIO(c.run().to_csv()), sep=",")
                data_frame.to_csv(path, index=False, compression='infer')
                cache.store_query(os.path.dirname(path), date_ini, date_end, "study_0", [q_entry], page)
            data_frames_page.append(data_frame)
        data_frame = pd.concat(data_frames_page, ignore_index=True, sort=False)

        return data_frame

    def retrieve_filtered(self, date_ini, date_end, q_entry):
        """
        Given the time interval and a query entry create a data frame with the result, filtered by similar values. The
        request result is cached in disk.

        :param date_ini: The search request initial date and time.
        :param date_end: The search request end date and time.
        :param q_entry: The search request query entry.
            {
                "component": "MACS.AzimuthAxis",
                "monitor": "position",
                "epsilon": 0.5,
                "type": "monitor"
            }

        :return: a data frame for this request, filtering the similar values base on epsilon monitor param.
        """
        monitor_key = q_entry['component'] + "." + q_entry['monitor']

        logging.info("Retrieve monitor %s from %s to %s ",
                     monitor_key, date_ini, date_end)

        path = cache.make_path_filtered(self._path, date_ini, date_end, monitor_key)
        logging.info("Retrieve monitor path %s ", os.path.dirname(path))
        if not exists(os.path.dirname(path)):
            os.makedirs(os.path.dirname(path))

        if exists(os.path.dirname(path) + "/query.json") and \
                cache.compare_filtered_query(cache.build_query(date_ini, date_end, self._query_name, [q_entry]),
                                             cache.read_query(os.path.dirname(path))):
            logging.info("Retrieve monitor from cache %s ", path)
            data_frame = pd.read_csv(path, compression='infer')
        else:
            logging.info("Retrieve monitor from API call")
            data_frame = self.retrieve_raw(date_ini, date_end, q_entry)
            if q_entry["type"] == "monitor" or q_entry["type"] == "array":
                data_frame = _remove_similar_consecutive_values(data_frame,
                                                            q_entry['component'] + "." + q_entry['monitor'],
                                                            q_entry["epsilon"])
                data_frame.to_csv(path, index=False, compression='infer')
                cache.store_query(os.path.dirname(path), date_ini, date_end, "study_0", [q_entry])
            else:
                data_frame.to_csv(path, index=False, compression='infer')
                cache.store_query(os.path.dirname(path), date_ini, date_end, "study_0", [q_entry])

        return data_frame

    def retrieve_summary_hourly(self, date_ini, date_end):
        """
        todo
        :return:
        """
        logging.info("Retrieve hourly")

        logging.info("Retrieve hourly from %s to %s", date_ini, date_end)
        path = cache.make_path_summary_hourly(self._path, date_ini, date_end, self._query_name)
        logging.info("Retrieve path %s ", os.path.dirname(path))
        if not exists(os.path.dirname(path)):
            os.makedirs(os.path.dirname(path))
        if exists(os.path.dirname(path) + "/query.json") and \
                cache.compare_summary_query(cache.build_query(date_ini, date_end, self._query_name, self._query),
                                            cache.read_query(os.path.dirname(path))):
            logging.info("Retrieve monitor from cache %s ", path)
            data_frame = pd.read_csv(path, compression='infer')
        else:
            logging.info("Retrieve monitor from api call")
            data_frames_hourly = []
            for idx, monitor in enumerate(self._query):
                logging.info("Retrieve %s %s %s ", date_ini, date_end, monitor)
                data_frame = self.retrieve_filtered(date_ini, date_end, monitor)
                data_frame = _convert(data_frame)
                data_frame = _filter(data_frame, date_ini, date_end)
                data_frames_hourly.append(data_frame)
            logging.info("Merge hours %s %s ", date_ini, date_end)
            data_frame = _merge_data_frames(data_frames_hourly)
            data_frame.to_csv(path, index=False, compression='infer')
            cache.store_query(os.path.dirname(path), date_ini, date_end, "study_0", self._query)
        return data_frame

    def retrieve_summary(self, date_ini, date_end):
        logging.info("Retrieve summary")

        path = cache.make_path_summary_query(self._path, date_ini, date_end, self._query_name)
        logging.info("Retrieve path %s ", os.path.dirname(path))

        if not exists(os.path.dirname(path)):
            os.makedirs(os.path.dirname(path))
        if exists(os.path.dirname(path) + "/query.json") and \
                cache.compare_summary_query(cache.build_query(date_ini, date_end,
                                                              self._query_name, self._query),
                                            cache.read_query(os.path.dirname(path))):
            logging.info("Retrieve monitor from cache %s ", path)

            data_frame = pd.read_csv(path, compression='infer')
        else:
            logging.info("Retrieve monitor from API call")

            data_frames_monitors = []
            time_intervals = _make_time_intervals(date_ini, date_end)
            for time_interval in time_intervals:
                logging.info("Retrieve hourly from %s to %s", time_interval[0], time_interval[1])

                data_frame = self.retrieve_summary_hourly(time_interval[0], time_interval[1])
                data_frames_monitors.append(data_frame)

            data_frame = _merge_data_frames(data_frames_monitors)
            data_frame.to_csv(path, index=False, compression='infer')
            cache.store_query(os.path.dirname(path), date_ini, date_end, "study_0", self._query)

        return data_frame

    def sanity_query_check(self, a_query):

        if not isinstance(a_query, (list, tuple)):
            raise "A query must be a list of monitors."
        else:
            for e in a_query:
                if not "component" in e: raise "Component not specified."
                if not "monitor" in e: raise "Monitor not specified."
                if not "epsilon" in e: raise "Epsilon not specified."
                if not "type" in e: raise "Type not specified."

                if not (type(e['epsilon']) == float or type(e['epsilon']) == int): raise "Epsilon should be float"
                if not e['type'] in ["monitor", "array", "magnitude"]:
                    raise "Not valid value for type should be: monitor, array or enum"

                try:
                    config = self.request.get_monitor_configuration(q_component=e['component'].replace('.', '/'),
                                                                    q_monitor=e['monitor'],
                                                                    q_type=e['type'])

                    if e['type'] in "array" and config["dimension_x"] == 1 and config["dimension_y"] == 1:
                        raise "Monitor dimension incorrect"
                except:
                    raise "Monitor does not exist in database."

    #todo Check if monitor if active. This can be problematic if monitor was active at some point but not now.
    #todo Inject the ID in the a_query. This mean later on it is not neccesary to ask for this value.
    #todo Inject the unit in case the user does not defined ¿for doing this i will add support for units in query?.
    #todo Inject epsilon in case the user does not defined, what happen if epsilon not in database, shall allow to not filter at all.
    #todo ¿include subsampling option, check in this case if subsampling query make sanse with sample of monitor?
    #todo ¿include range control for the values optinally? in this case populate ddbb correctly
    #todo do chache of summary by day and store informatio to know if necessary to reproduce.
    #todo Include a summary of the process of download.
    #todo include a progress bar
    #todo wildcard, all the active monitor for a device..
    #todo reemplazar los NaN por valores iguales, antes y despues....


from mmwebreport.core.executor import Executor

_QUERY_DESCRIPTION_MONITOR_TOKEN = "monitors"
_QUERY_DESCRIPTION_MAGNITUDE_TOKEN = "magnitud"

_QUERY_MONITOR_TOKEN = "idmonitor"
_QUERY_MAGNITUDE_TOKEN = "idmagnitud"

_QUERY_PAGE_START_TOKEN = "page"
_QUERY_PAGE_LENGTH_TOKEN = "length"

_DEFAULT_SAMPLES_PER_PAGE = 30000


class Cursor(object):
    """
    A class used to iterate over the pages of a request response from the Monitor Manager Web Report Backend.
    Each pages will consist of a maximum of _DEFAULT_SAMPLES_PER_PAGE samples.

    It is important to notice that the iterator does not execute the request, instead an Executor is returned that
    must be explicit invoked to run the query.
    """
    def __init__(self, url, pages, initial_page=0):
        """
        Initialize the iterator.

        :param url: The Http URL for the request to Monitor Manager Web report backend.
        :param pages: The number of pages for the request response.
        :param initial_page: The initial page number.
        """
        # todo check consistency, if the initial page is lower than the number of pages
        self._url = url
        self._pages = pages
        self._initial_page = initial_page
        self._current = initial_page

    def __iter__(self):
        """
        Initialize the current page to the initial page

        :return: The iterator itself
        """
        self._current = self._initial_page
        return self

    def __next__(self):
        """
        Add to URL the page information and return an Executor object that can be used to invoke the request.

        :return: Executor object to make the final invocation
        """
        if self._current >= self._pages:
            raise StopIteration

        url = self._url + \
              "&" + _QUERY_PAGE_START_TOKEN + "=" + str(self._current) + \
              "&" + _QUERY_PAGE_LENGTH_TOKEN + "=" + str(_DEFAULT_SAMPLES_PER_PAGE)

        self._current += 1

        return Executor(url)

    def size(self):
        """
        Accessor method to the size of iterator

        :return:  the number of pages.
        """
        return self._pages


class Report(object):
    """
    """
    def __init__(self, host, port):
        self._host = host
        self._port = port
        self._base_url = "http://" + self._host + ":" + self._port + "/WebReport/rest"

    def get_components(self):
        """
        Return the list of components
        """
        url = self._base_url
        url = url + "/components"

        executor = Executor(url)
        response = executor.run()

        return response.to_json()

    def get_component(self, q_component):
        """
        Given a component, return monitor and magnitudes for this component.

        :param q_component: the component name

        :return: the configuration of a component in json format. The output format is:

            {
                'id': ''
                'name': ''
                'className': ''
                'magnitudeDescription': []
                'monitorDescription' : []
            }

        """
        url = self._base_url
        url = url + "/components/"
        url = url + q_component

        executor = Executor(url)
        response = executor.run()

        return response.to_json()

    def get_monitor_configuration(self, q_component, q_monitor, q_type="monitor"):
        """
        Given a component and a monitor return the monitor configuration

        :param q_component: the component name.
        :param q_monitor: the monitor name
        :param q_type: the type, it can be monitor or magnitude

        :return: the monitor configuration in json format. The output format is:

            {
                'id': ''
                'magnitude: ''
                'dimension_x' : ''
                'dimension_y' : ''
                'description' : ''
                'type' : ''
                'unit' : ''
                'config' :  {
                                'id': ''
                                'active' : ''
                                'epsilon' : ''
                                'persite_by_default' : ''
                                'propagate_period' : ''
                                'storage_period' : ''
                                'monitorRanges' : {...}
                            }
            }
        """
        query_parsed = _QUERY_DESCRIPTION_MONITOR_TOKEN
        if q_type == "magnitude":
            query_parsed = _QUERY_DESCRIPTION_MAGNITUDE_TOKEN

        url = self._base_url
        url = url + "/components/"
        url = url + q_component + "/" + query_parsed + "/" + q_monitor

        executor = Executor(url)
        response = executor.run()

        return response.to_json()

    def search_info(self, q_data_ini, q_data_end, q_component, q_monitor, q_type="monitor"):
        """
        Given the parameters of a search request, return the metadata information about this query.

        :param q_data_ini: The search request initial date and time.
        :param q_data_end: The search request end date and time.
        :param q_component: The search request component.
        :param q_monitor: The search request monitor of the component.
        :param q_type: The search request type, it can be monitor or mangnitude.

        :return: the search request metadata in json format. The output format is:
                {
                    'length': ''
                    'totalPages: ''
                    'totalSamples' : ''
                }

        """
        query = [{"component": q_component, "monitor": q_monitor, "epsilon": 0.5, "type": q_type}]
        return self._search_info(q_data_ini, q_data_end, query)

    def search(self, q_data_ini, q_data_end, q_query):
        """
        Given the temporal parameters of a search request, and the query itself, return a Cursor to iterator over
        the query result. It is important to notice that the iterator does not execute the request, instead an Executor
        is returned that must be explicit invoked to run the query.

        :param q_data_ini: The search request initial date and time.
        :param q_data_end: The search request end date and time.
        :param q_query: The query itself has the following format:
            query =
                [
                    {
                        "component": "",
                        "monitor": "",
                        "epsilon": ,
                        "type": ""
                    }
                ]

        :return: a :Cursor to iterate over the result.
        """
        search_monitor_description \
            = self._search_info(q_data_ini, q_data_end, q_query)

        url = self._base_url
        url = url + "/download"
        url = url + self._parse_search(q_data_ini, q_data_end, sampling=0)
        url = url + self._parse_monitors(q_query)

        a_cursor = Cursor(url, search_monitor_description["totalPages"])

        return a_cursor

    def search_stored_info(self, q_data_ini, q_data_end, q_query_name):
        """
        Given the temporal parameters of a search request, and the query stored name, return the metadata
        information about this query.

        :param q_data_ini: The search request initial date and time.
        :param q_data_end: The search request end date and time.
        :param q_query_name: The query stored name. This query must exits, and it can be created with the Monitor
        Manager Web Report frontend.

        :return: the search request metadata in json format. The output format is:
                {
                    'length': ''
                    'totalPages: ''
                    'totalSamples' : ''
                }
        """
        url = self._base_url
        url = url + "/query/metadata/"
        url = url + q_query_name
        url = url + self._parse_search(q_data_ini, q_data_end)
        url = url + "&" + _QUERY_PAGE_START_TOKEN + "=" + str(0)
        url = url + "&" + _QUERY_PAGE_LENGTH_TOKEN + "=" + str(_DEFAULT_SAMPLES_PER_PAGE)

        executor = Executor(url)
        response = executor.run()

        return response.to_json()

    def search_stored_query(self, q_data_ini, q_data_end, q_query_name):
        """
        Given the temporal parameters of a search request, and the query stored name, return a Cursor to iterator over
        the query result. It is important to notice that the iterator does not execute the request, instead an Executor
        is returned that must be explicit invoked to run the query.

        :param q_data_ini: The search request initial date and time.
        :param q_data_end: The search request end date and time.
        :param q_query_name: The query stored name. This query must exits and it can be created with the Monitor
        Manager Web Rerpot frontend.

        :return: a :Cursor to iterate over the result.
        """
        search_monitor_description \
            = self.search_stored_info(q_data_ini, q_data_end, q_query_name)

        url = self._base_url
        url = url + "/query/download/"
        url = url + q_query_name
        url = url + self._parse_search(q_data_ini, q_data_end)
        url = url + "&" + _QUERY_PAGE_START_TOKEN + "=" + str(0)
        url = url + "&" + _QUERY_PAGE_LENGTH_TOKEN + "=" + str(_DEFAULT_SAMPLES_PER_PAGE)

        a_cursor = Cursor(url, search_monitor_description["totalPages"])

        return a_cursor

    def _parse_search(self, q_date_ini, q_date_end, sampling=None):
        """
        Build the URL part of a request related to the temporal part.

        :param q_date_ini: The search request initial date and time.
        :param q_date_end: The search request end date and time.
        :param sampling: The sampling period, of the moment only the default 0 is supported.

        :return: a string with the request related to the temporal part.

                The format is: /01/03/2022@23:50:00.000/02/03/2022@00:00:00.000/0?

        """
        search_uri_part = ""
        search_uri_part = search_uri_part + q_date_ini.strftime("/%d/%m/%Y@%H:%M:%S.%f")[:-3]
        search_uri_part = search_uri_part + q_date_end.strftime("/%d/%m/%Y@%H:%M:%S.%f")[:-3]
        if sampling is not None:
            search_uri_part = search_uri_part + "/" + str(sampling) + "?"
        else:
            search_uri_part = search_uri_part + "?"

        return search_uri_part

    def _parse_single_monitor(self, monitor, q_type):
        """
        Build the URL part of a request related to monitor.

        :param monitor: The search request monitor name of the component.
        :param q_type: The search request type, it can be monitor or mangnitude.

        :return: a string with the request related to the monitor part.

            The format is: idmonitor=3625
        """
        # todo exception if q_type not found
        query_parsed = ""
        if q_type == "monitor":
            query_parsed = _QUERY_MONITOR_TOKEN + "=" + str(monitor["id"])
        elif q_type == "magnitude":
            query_parsed = _QUERY_MAGNITUDE_TOKEN + "=" + str(monitor["id"])
        elif q_type == "array":
            query_parsed = _QUERY_MONITOR_TOKEN + "=" + str(monitor["id"]) + "[[-1]]"
        else:
            raise "Type not supported"

        return query_parsed

    def _parse_monitors(self, a_query):
        """
        Build the URL part of a request related to a set of monitor.

        :param a_query: The query itself has the following format:
            query =
                [
                    {
                        "component": "",
                        "monitor": "",
                        "epsilon": ,
                        "type": ""
                    }
                ]
        :return: a string with the request related to a monitor set part.

            The format is: idmonitor=3623&idmonitor=3625

        """
        query_monitor_uri = ""

        if isinstance(a_query, (list, tuple)):
            for q in a_query:
                m = self.get_monitor_configuration(q["component"].replace('.', '/'), q["monitor"], q["type"])
                query_monitor_uri = query_monitor_uri + self._parse_single_monitor(m, q["type"]) + "&"
            query_monitor_uri = query_monitor_uri[:-1]
        else:
            m = self.get_monitor_configuration(a_query["component"].replace('.', '/'), a_query["monitor"], a_query["type"])

            query_monitor_uri = self._parse_single_monitor(a_query, m["type"])

        return query_monitor_uri

    def _search_info(self, q_data_ini, q_data_end, q_query):
        """
        Build the URL to make a request to obtain the metadata information about a query.

        :param q_data_ini: The search request initial date and time.
        :param q_data_end: The search request end date and time.
        :param q_query: The query itself has the following format:
            query =
                [
                    {
                        "component": "",
                        "monitor": "",
                        "epsilon": ,
                        "type": ""
                    }
                ]
        :return:  a string with the request URL to obtain metadata info for a query

            The format is: idmonitor=3623&idmonitor=3625
                .../search/metadata/01/03/2022@20:00:00.000/01/03/2022@20:00:10.000/0?
                        idmonitor=3623&
                        idmonitor=3625&
                        ...
                        length=30000

        """

        url = self._base_url + "/search/metadata"
        url = url + self._parse_search(q_data_ini, q_data_end, sampling=0)
        url = url + self._parse_monitors(q_query)
        url = url + "&" + _QUERY_PAGE_LENGTH_TOKEN + "=" + str(_DEFAULT_SAMPLES_PER_PAGE)

        executor = Executor(url)
        response = executor.run()

        return response.to_json()

    def single_search(self, q_data_ini, q_data_end, q_component, q_monitor, q_type="monitor"):
        """
        Given the temporal parameters of a search request, and the query itself, return a Cursor to iterator over
        the query result. It is important to notice that the iterator does not execute the request, instead an Executor
        is returned that must be explicit invoked to run the query.

        :param q_data_ini: The search request initial date and time.
        :param q_data_end: The search request end date and time.
        :param q_component: The search request component name.
        :param q_monitor: The search request monitor name.
        :param q_type: The search request type, it can be monitor or mangnitude.

        :return: a :Cursor to iterate over the result.
        """
        query = [{"component": q_component, "monitor": q_monitor, "epsilon": 0.5, "type": q_type}]
        return self.search(q_data_ini, q_data_end, query)



import setuptools

setuptools.setup(
    name='mmwebreportapi',
    packages=['mmwebreport','mmwebreport.core','mmwebreport.retrieve'],
    version='0.0.1',
    description='Monitor Manager Web Rerpot API',
    author='Manuel Huertas Lopez',
    author_email='manuel.huertas.lopez@gmail.com',
    install_requires=['pandas', 'requests'],
    download_url="https://github.com/mahulo2009/monitormanager-webreportapi-python/archive/refs/tags/v1.0.0-alpha.tar.gz"
)


import os
import json

from datetime import datetime, time, timedelta


def make_path_raw(root_path, date_ini, date_end, a_id, page=None):
    path = root_path + "/" + \
           date_ini.strftime("%Y-%m-%d") + "/" + date_ini.strftime("%H") + \
           "/" + str(a_id).replace(".", "/") + "/raw"

    file_name_monitor = str(a_id) + \
                        "." + \
                        date_ini.strftime("%Y-%m-%d_%H_%M_%S") + \
                        "." + \
                        date_end.strftime("%Y-%m-%d_%H_%M_%S")
    if page is None:
        file_name_monitor = file_name_monitor + ".gz"
    else:
        file_name_monitor = file_name_monitor + ".raw." + str(page).rjust(4, '0') + ".gz"

    return path + "/" + file_name_monitor


def make_path_filtered(root_path, date_ini, date_end, a_id):
    path = root_path + "/" + \
           date_ini.strftime("%Y-%m-%d") + "/" + date_ini.strftime("%H") + \
           "/" + str(a_id).replace(".", "/")

    file_name_monitor = str(a_id) + \
                        "." + \
                        date_ini.strftime("%Y-%m-%d_%H_%M_%S") + \
                        "." + \
                        date_end.strftime("%Y-%m-%d_%H_%M_%S")
    file_name_monitor = file_name_monitor + ".gz"

    return path + "/" + file_name_monitor


def make_path_summary_hourly(root_path, date_ini, date_end, query_name):
    path = root_path + "/" \
           + date_ini.strftime("%Y-%m-%d") + "/" + date_ini.strftime("%H") \
           + "/summary"

    file_name_monitor = date_ini.strftime("%Y-%m-%d_%H_%M_%S") + \
                        "." + \
                        date_end.strftime("%Y-%m-%d_%H_%M_%S")

    file_name_monitor = query_name + "." + "merge" + "." + file_name_monitor + ".gz"

    return path + "/" + file_name_monitor


def store_query_summary_hourly(root_path, date_ini, date_end, query_name, query):
    query = {
        "date_ini": date_ini.strftime("%Y-%m-%d %H_%M_%S"),
        "date_end": date_end.strftime("%Y-%m-%d %H_%M_%S"),
        "query_name": query_name,
        "query": query
    }

    with open(root_path + '/query.json', 'w') as outfile:
        outfile.write(json.dumps(query))


def make_path_summary_query(root_path, date_ini, date_end, query_name):
    path = root_path + "/summary/" + query_name

    file_name_monitor = date_ini.strftime("%Y-%m-%d_%H_%M_%S") + \
                        "." + \
                        date_end.strftime("%Y-%m-%d_%H_%M_%S")
    file_name_monitor = file_name_monitor + ".summary" + ".gz"

    return path + "/" + file_name_monitor


def store_query(root_path, date_ini, date_end, query_name, query, page=''):
    query = {
        "date_ini": date_ini.strftime("%Y-%m-%d %H_%M_%S"),
        "date_end": date_end.strftime("%Y-%m-%d %H_%M_%S"),
        "query_name": query_name,
        "query": query
    }

    if page is not None:
        query["page"] = page

    with open(root_path + '/query'+str(page)+'.json', 'w') as outfile:
        outfile.write(json.dumps(query))


def build_query(date_ini, date_end, query_name, query, page=None):
    q = {
        "date_ini": date_ini.strftime("%Y-%m-%d %H_%M_%S"),
        "date_end": date_end.strftime("%Y-%m-%d %H_%M_%S"),
        "query_name": query_name,
        "query": query
    }

    if page is not None:
        q["page"] = page

    return q


def read_query(path, page=''):
    with open(path + "/query" + str(page) + ".json") as json_file:
        return json.load(json_file)


def compare_raw_query(q1, q2, index=0):
    equal = True

    equal = equal and (q1["date_ini"] == q2["date_ini"])
    equal = equal and (q1["date_end"] == q2["date_end"])
    if len(q1["query"]) > 0:
        equal = equal and (q1["query"][index]["component"] == q2["query"][index]["component"])
        equal = equal and (q1["query"][index]["monitor"] == q2["query"][index]["monitor"])
        equal = equal and (q1["query"][index]["type"] == q2["query"][index]["type"])

    #    if q1["page"] and q2["page"]:
    #        equal = equal and (q1["page"] == q2["page"])

    return equal


def compare_filtered_query(q1, q2, index=0):
    equal = compare_raw_query(q1, q2, index)

    if len(q1["query"]) > 0:
        equal = equal and (q1["query"][index]["epsilon"] == q2["query"][index]["epsilon"])

    return equal


def compare_summary_query(q1, q2):
    equal = len(q1["query"]) == len(q2["query"])
    for index in range(0, len(q1["query"])):
        equal = equal and compare_filtered_query(q1, q2, index)

    return equal
